@@ -29,7 +29,6 @@
 XTRACE=$(set +o | grep xtrace)
 set +o xtrace
 
-
 # Functions
 # ---------
 
@@ -46,6 +45,7 @@ function cleanup_zaqar {
     if [ "$ZAQAR_BACKEND" = 'mongodb' ] ; then
         cleanup_zaqar_mongodb
     fi
+    remove_uwsgi_config "$ZAQAR_UWSGI_CONF" "zaqar"
 }
 
 # cleanup_zaqar_mongodb() - Remove residual data files, anything left over from previous
@@ -81,7 +81,7 @@ function configure_zaqar {
     iniset $ZAQAR_CONF DEFAULT debug True
     iniset $ZAQAR_CONF DEFAULT unreliable True
     iniset $ZAQAR_CONF DEFAULT admin_mode True
-    iniset $ZAQAR_CONF DEFAULT enable_deprecated_api_versions 1,1.1
+    iniset $ZAQAR_CONF DEFAULT enable_deprecated_api_versions 1.1
     iniset $ZAQAR_CONF signed_url secret_key notreallysecret
 
     if is_service_enabled key; then
@@ -96,7 +96,7 @@ function configure_zaqar {
     iniset $ZAQAR_CONF 'drivers:transport:websocket' port $ZAQAR_WEBSOCKET_PORT
     iniset $ZAQAR_CONF drivers transport websocket
 
-    configure_auth_token_middleware $ZAQAR_CONF zaqar $ZAQAR_AUTH_CACHE_DIR
+    configure_keystone_authtoken_middleware $ZAQAR_CONF zaqar
 
     iniset $ZAQAR_CONF trustee auth_type password
     iniset $ZAQAR_CONF trustee auth_url $KEYSTONE_AUTH_URI
@@ -107,10 +107,6 @@ function configure_zaqar {
     iniset $ZAQAR_CONF DEFAULT pooling True
     iniset $ZAQAR_CONF 'pooling:catalog' enable_virtual_pool True
 
-    # NOTE(flaper87): Configure mongodb regardless so we can use it as a pool
-    # in tests.
-    configure_mongodb
-
     if [ "$ZAQAR_BACKEND" = 'mongodb' ] ; then
         iniset $ZAQAR_CONF  drivers message_store mongodb
         iniset $ZAQAR_CONF 'drivers:message_store:mongodb' uri mongodb://localhost:27017/zaqar
@@ -119,13 +115,14 @@ function configure_zaqar {
         iniset $ZAQAR_CONF  drivers management_store mongodb
         iniset $ZAQAR_CONF 'drivers:management_store:mongodb' uri mongodb://localhost:27017/zaqar_mgmt
         iniset $ZAQAR_CONF 'drivers:management_store:mongodb' database zaqar_mgmt
+        configure_mongodb
     elif [ "$ZAQAR_BACKEND" = 'redis' ] ; then
         recreate_database zaqar
         iniset $ZAQAR_CONF  drivers management_store sqlalchemy
         iniset $ZAQAR_CONF 'drivers:management_store:sqlalchemy' uri `database_connection_url zaqar`
         iniset $ZAQAR_CONF 'drivers:management_store:sqlalchemy' database zaqar_mgmt
 
-        zaqar-sql-db-manage --config-file $ZAQAR_CONF upgrade head
+        $ZAQAR_BIN_DIR/zaqar-sql-db-manage --config-file $ZAQAR_CONF upgrade head
 
         iniset $ZAQAR_CONF  drivers message_store redis
         iniset $ZAQAR_CONF 'drivers:message_store:redis' uri redis://localhost:6379
@@ -137,7 +134,7 @@ function configure_zaqar {
         iniset $ZAQAR_CONF 'drivers:management_store:sqlalchemy' uri `database_connection_url zaqar`
         iniset $ZAQAR_CONF 'drivers:management_store:sqlalchemy' database zaqar_mgmt
 
-        zaqar-sql-db-manage --config-file $ZAQAR_CONF upgrade head
+        $ZAQAR_BIN_DIR/zaqar-sql-db-manage --config-file $ZAQAR_CONF upgrade head
 
         iniset $ZAQAR_CONF  drivers message_store swift
         iniset $ZAQAR_CONF 'drivers:message_store:swift' auth_url $KEYSTONE_AUTH_URI
@@ -150,23 +147,7 @@ function configure_zaqar {
     fi
     iniset_rpc_backend zaqar $ZAQAR_CONF DEFAULT
 
-    pip_install uwsgi
-
-    iniset $ZAQAR_UWSGI_CONF uwsgi master true
-    iniset $ZAQAR_UWSGI_CONF uwsgi die-on-term true
-    iniset $ZAQAR_UWSGI_CONF uwsgi exit-on-reload true
-    iniset $ZAQAR_UWSGI_CONF uwsgi http $ZAQAR_SERVICE_HOST:$ZAQAR_SERVICE_PORT
-    iniset $ZAQAR_UWSGI_CONF uwsgi processes $API_WORKERS
-    iniset $ZAQAR_UWSGI_CONF uwsgi enable_threads true
-    iniset $ZAQAR_UWSGI_CONF uwsgi threads 4
-    iniset $ZAQAR_UWSGI_CONF uwsgi thunder-lock true
-    iniset $ZAQAR_UWSGI_CONF uwsgi buffer-size 65535
-    iniset $ZAQAR_UWSGI_CONF uwsgi wsgi-file $ZAQAR_DIR/zaqar/transport/wsgi/app.py
-    iniset $ZAQAR_UWSGI_CONF uwsgi master true
-    iniset $ZAQAR_UWSGI_CONF uwsgi add-header "Connection: close"
-    iniset $ZAQAR_UWSGI_CONF uwsgi lazy-apps true
-
-    cleanup_zaqar
+    write_uwsgi_config "$ZAQAR_UWSGI_CONF" "$ZAQAR_UWSGI" "/messaging" "" "zaqar"
 }
 
 function configure_redis {
@@ -186,15 +167,22 @@ function configure_mongodb {
     # per database.
     pip_install pymongo
     if is_ubuntu; then
-        # NOTE: To fix the mongodb's issue in ubuntu 22.04 LTS
+        # NOTE: To fix the mongodb's issue in ubuntu 22.04/24.04 LTS
         ubuntu_version=$(source /etc/os-release ; echo $VERSION_ID)
-        if [[ $ubuntu_version == '22.04' ]]; then
+        if [[ $ubuntu_version == '24.04' ]]; then
+            if [[ ! -d /etc/apt/sources.list.d ]]; then
+                sudo mkdir -p /etc/apt/sources.list.d
+            fi
+            wget -qO - https://www.mongodb.org/static/pgp/server-8.0.asc | sudo apt-key add -
+            echo "deb [ arch=amd64,arm64 ] https://repo.mongodb.org/apt/ubuntu noble/mongodb-org/8.0 multiverse" | sudo tee /etc/apt/sources.list.d/mongodb-org-8.0.list
+            sudo apt update
+            install_package mongodb-org
+            restart_service mongod
+            sudo systemctl status mongod
+        elif [[ $ubuntu_version == '22.04' ]]; then
             wget -qO - https://www.mongodb.org/static/pgp/server-7.0.asc | sudo apt-key add -
             echo "deb [ arch=amd64,arm64 ] https://repo.mongodb.org/apt/ubuntu jammy/mongodb-org/7.0 multiverse" | sudo tee /etc/apt/sources.list.d/mongodb-org-7.0.list
-            echo "deb http://security.ubuntu.com/ubuntu bionic-security main" | sudo tee /etc/apt/sources.list.d/bionic-security.list
-            sudo apt-key adv --keyserver keyserver.ubuntu.com --recv-keys 3B4FE6ACC0B21F32
             sudo apt update
-            install_package libssl1.1
             install_package mongodb-org
             restart_service mongod
             sudo systemctl status mongod
@@ -225,10 +213,8 @@ __EOF__
 
 # init_zaqar() - Initialize etc.
 function init_zaqar {
-    # Create cache dir
-    sudo mkdir -p $ZAQAR_AUTH_CACHE_DIR
-    sudo chown $STACK_USER $ZAQAR_AUTH_CACHE_DIR
-    rm -f $ZAQAR_AUTH_CACHE_DIR/*
+    # Nothing to do
+    :
 }
 
 # install_zaqar() - Collect source and prepare
@@ -238,6 +224,7 @@ function install_zaqar {
     if is_service_enabled horizon; then
         install_zaqarui
     fi
+    pip_install uwsgi
 }
 
 function install_zaqarui {
@@ -266,14 +253,14 @@ function install_zaqarclient {
 
 # start_zaqar() - Start running processes, including screen
 function start_zaqar {
-    cat $ZAQAR_UWSGI_CONF
-    run_process zaqar-wsgi "$ZAQAR_BIN_DIR/uwsgi --ini $ZAQAR_UWSGI_CONF --pidfile2 $ZAQAR_UWSGI_MASTER_PIDFILE"
+    run_process zaqar-wsgi "$ZAQAR_BIN_DIR/uwsgi --ini $ZAQAR_UWSGI_CONF"
     run_process zaqar-websocket "$ZAQAR_BIN_DIR/zaqar-server --config-file $ZAQAR_CONF"
 
     echo "Waiting for Zaqar to start..."
     local www_authenticate_uri=http://${ZAQAR_SERVICE_HOST}/identity
+    local ping_url=$ZAQAR_SERVICE_PROTOCOL://$ZAQAR_SERVICE_HOST/messaging/v2/ping
     token=$(openstack token issue -c id -f value --os-auth-url ${www_authenticate_uri})
-    if ! timeout $SERVICE_TIMEOUT sh -c "while ! wget --no-proxy -q --header=\"Client-ID:$(uuidgen)\" --header=\"X-Auth-Token:$token\" -O- $ZAQAR_SERVICE_PROTOCOL://$ZAQAR_SERVICE_HOST:$ZAQAR_SERVICE_PORT/v2/ping; do sleep 1; done"; then
+    if ! timeout $SERVICE_TIMEOUT sh -c "while ! wget --no-proxy -q --header=\"Client-ID:$(uuidgen)\" --header=\"X-Auth-Token:$token\" -O- $ping_url; do sleep 1; done"; then
         die $LINENO "Zaqar did not start"
     fi
 }
@@ -283,9 +270,8 @@ function stop_zaqar {
     local serv
     # Kill the zaqar screen windows
     for serv in zaqar-wsgi zaqar-websocket; do
-        screen -S $SCREEN_NAME -p $serv -X kill
+        stop_process serv
     done
-    uwsgi --stop $ZAQAR_UWSGI_MASTER_PIDFILE
 }
 
 function create_zaqar_accounts {
@@ -293,20 +279,15 @@ function create_zaqar_accounts {
 
     if [[ "$KEYSTONE_IDENTITY_BACKEND" = 'sql' ]]; then
 
-        local zaqar_service=$(get_or_create_service "zaqar" \
-            "messaging" "Zaqar Service")
-        get_or_create_endpoint $zaqar_service \
+        get_or_create_service "zaqar" "messaging" "Zaqar Service"
+        get_or_create_endpoint "messaging" \
             "$REGION_NAME" \
-            "$ZAQAR_SERVICE_PROTOCOL://$ZAQAR_SERVICE_HOST:$ZAQAR_SERVICE_PORT" \
-            "$ZAQAR_SERVICE_PROTOCOL://$ZAQAR_SERVICE_HOST:$ZAQAR_SERVICE_PORT" \
-            "$ZAQAR_SERVICE_PROTOCOL://$ZAQAR_SERVICE_HOST:$ZAQAR_SERVICE_PORT"
+            "$ZAQAR_SERVICE_PROTOCOL://$ZAQAR_SERVICE_HOST/messaging"
 
-        local zaqar_ws_service=$(get_or_create_service "zaqar-websocket" \
-            "messaging-websocket" "Zaqar Websocket Service")
-        get_or_create_endpoint $zaqar_ws_service \
+        get_or_create_service "zaqar-websocket" \
+            "messaging-websocket" "Zaqar Websocket Service"
+        get_or_create_endpoint "messaging-websocket" \
             "$REGION_NAME" \
-            "$ZAQAR_SERVICE_PROTOCOL://$ZAQAR_SERVICE_HOST:$ZAQAR_WEBSOCKET_PORT" \
-            "$ZAQAR_SERVICE_PROTOCOL://$ZAQAR_SERVICE_HOST:$ZAQAR_WEBSOCKET_PORT" \
             "$ZAQAR_SERVICE_PROTOCOL://$ZAQAR_SERVICE_HOST:$ZAQAR_WEBSOCKET_PORT"
     fi
 
@@ -338,6 +319,10 @@ if is_service_enabled zaqar-websocket || is_service_enabled zaqar-wsgi; then
     if [[ "$1" == "unstack" ]]; then
         stop_zaqar
     fi
+
+    if [[ "$1" == "clean" ]]; then
+        cleanup_zaqar
+    fi
 fi
 
 # Restore xtrace
